/*---------------------------------------------------------------------------* 
 * File:        udp_comm_spoofed.h                                           *  
 * Descrizione: Routines per la comunicazione attraverso il protocollo UDP   *  
 *              con spoofing degli indirizzi IP.                             *
 * Revisione:   v1.0                                                         *  
 *                                                                           *  
 * NO(C)2K MegaBug                                                           *  
 *---------------------------------------------------------------------------*/

#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <arpa/inet.h>
#include <unistd.h>


/*---------------------------------------------------------------------------*
 * Apre una raw socket.                                                      *
 *---------------------------------------------------------------------------*/
int ApriRAW( void )
{
	int rawSock, sockOpt;
		
	/* Apre una RAW socket */
        rawSock = socket( AF_INET, SOCK_RAW, IPPROTO_RAW );
        if ( rawSock == -1 )
                return -1;
	
	/* forniamo noi tutti gli header dei pacchetti... */
	sockOpt = 1;
        if ( setsockopt( rawSock, IPPROTO_IP, IP_HDRINCL, (char *)&sockOpt, sizeof(sockOpt) ) == -1 )
                return -2;

	return rawSock;
}

void ChiudiRAW( int rawSock )
{
	close( rawSock );
}

/*---------------------------------------------------------------------------*
 * Invia un pacchetto UDP a Indirizzo/Porta                                  *
 * L'indirizzo sorgente e' spoofato. Viene sostituito con Sorg.              *
 * Se l'indirizzo non  numerico NON viene risolto e la funzione FALLISCE!   *
 *---------------------------------------------------------------------------*/
int InviaUDPSpoofed(
    int rawSock,
    u_long Sorg, int SPorta,
    u_long Dest, int DPorta,
    char *buffer, int len
    )
{
	const int MAXDATA = 0x200;	/* Dimensione massima dei dati trasmessi */
	
	struct sockaddr_in remote_addr;
	struct
	{
	        struct iphdr ip;
	        struct udphdr udp;
	        char data[MAXDATA];
        } Pacchetto;
        
	/* Calcola la dimensione TOTALE del pacchetto (IP+UDP) */
	int psize = sizeof(struct udphdr) + sizeof(struct iphdr) + len;

	/* Riempie l'header IP */
        Pacchetto.ip.version = 4;
        Pacchetto.ip.ihl = 5;
        Pacchetto.ip.tot_len = htons( psize );
        Pacchetto.ip.id = htons( 0x0455 );
        Pacchetto.ip.ttl = 64;
        Pacchetto.ip.protocol = IPPROTO_UDP;
        Pacchetto.ip.saddr = Sorg;
        Pacchetto.ip.daddr = Dest;
        Pacchetto.ip.frag_off = htons( 0x0000 );	/* IP NON frammentato */

	/* Riempie l'header UDP */
        Pacchetto.udp.source = htons( SPorta );
        Pacchetto.udp.dest = htons( DPorta );
        Pacchetto.udp.len = htons( 8 + len );

	/* Riempie il campo dati */
	if ( len > MAXDATA ) return -1;
	memcpy( &Pacchetto.data, buffer, len );

	/* Invia il pacchetto */	
        remote_addr.sin_family = AF_INET;        
        remote_addr.sin_port = htons( DPorta );
        remote_addr.sin_addr.s_addr = Dest;
        return sendto( rawSock, &Pacchetto, psize, 0, (struct sockaddr *)&remote_addr, sizeof(remote_addr) );

}

