/*---------------------------------------------------------------------------* 
 * File:        udp_comm.h                                                   *  
 * Descrizione: Routines per la comunicazione attraverso il protocollo UDP.  *  
 * Revisione:   v1.0                                                         *  
 *                                                                           *  
 * NO(C)2K MegaBug                                                           *  
 *---------------------------------------------------------------------------*/

#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

/*---------------------------------------------------------------------------* 
 * Restituisce il socket aperto per ricevere dati dalla porta indicata       *  
 *---------------------------------------------------------------------------*/
int ApriUDP( int Porta )
{
	int udpSock;
	static struct sockaddr_in serv_addr;

	/* Apre la socket in modalita' UDP */
	udpSock = socket( AF_INET, SOCK_DGRAM, 0 );
	if ( udpSock == -1 )
		return -1;

	/* Aggancia la socket alla porta specificata */
	serv_addr.sin_addr.s_addr = htonl( INADDR_ANY );
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_port = htons( Porta );
	if ( bind( udpSock, (struct sockaddr *) &serv_addr, sizeof(serv_addr) ) == -1 )
		return -2;

	return udpSock;
}

void ChiudiUDP( int udpSock )
{
	close( udpSock );
}

/*---------------------------------------------------------------------------*
 * Invia un pacchetto UDP a Indirizzo/Porta                                  *
 * Se l'indirizzo non  numerico NON viene risolto e la funzione FALLISCE!   *
 *---------------------------------------------------------------------------*/
int InviaUDP( int udpSock, u_long Indirizzo, int Porta, char *buffer, int len )
{
	struct sockaddr_in remote_addr;

	/* Prepara l'indirizzo e invia il pacchetto */
	remote_addr.sin_addr.s_addr = Indirizzo;
	remote_addr.sin_family = AF_INET;
	remote_addr.sin_port = htons( Porta );
	return sendto( udpSock, buffer, len, 0, (struct sockaddr *) &remote_addr, sizeof(remote_addr) );
}

/*---------------------------------------------------------------------------*
 * Riceve un pacchetto UDP                                                   *
 * Restituisce l'indirizzo del mittente in RetAddr                           *
 *---------------------------------------------------------------------------*/
int RiceviUDP( int udpSock, char *buffer, int len, u_long RetAddr)
{
	struct sockaddr_in remote_addr;
	int res;
	int addr_len;
	
	/* Legge il pacchetto dalla socket e copia l'indirizzo remoto */
	addr_len = sizeof( struct sockaddr );
	res = recvfrom( udpSock, buffer, len, 0, (struct sockaddr*) &remote_addr, &addr_len ); 
	RetAddr = remote_addr.sin_addr.s_addr;
	return res;
}
